-- Filename:     tb_kookwekker_counter.vhd
-- Filetype:     VHDL Testbench
-- Date:         23 may 2015
-- Update:       -
-- Description:  Testbench for simulating the kookwekker counter
-- Author:       J. op den Brouw
-- State:        Demo
-- Error:        -
-- Version:      1.0
-- Copyright:    (c)2015, De Haagse Hogeschool

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- Empty entity
entity tb_kookwekker_counter is
end entity tb_kookwekker_counter;

-- The architecture...
architecture sim of tb_kookwekker_counter is

-- The component to test
component kookwekker_counter is
	port
	(
		clk_32768  : in std_logic; 
		areset     : in std_logic; 
		mins_up    : in std_logic; 
		secs_up    : in std_logic; 
		countdown  : in std_logic;
		clear      : in std_logic;
		tens_mins  : out unsigned(3 downto 0);
		mins       : out unsigned(3 downto 0);
		tens_secs  : out unsigned(3 downto 0);
		secs       : out unsigned(3 downto 0);
		time_zero  : out std_logic
	);
end component kookwekker_counter;


-- The signals to monitor...
signal sim_clk_32768  : std_logic; 
signal sim_areset     : std_logic; 
signal sim_mins_up    : std_logic; 
signal sim_secs_up    : std_logic; 
signal sim_countdown  : std_logic;
signal sim_clear      : std_logic;
signal sim_tens_mins  : unsigned(3 downto 0);
signal sim_mins       : unsigned(3 downto 0);
signal sim_tens_secs  : unsigned(3 downto 0);
signal sim_secs       : unsigned(3 downto 0);
signal sim_time_zero  : std_logic;

-- Synchronizing signal to stop the clock.
signal still_simulating : boolean := true;

-- How many minutes and seconds to count (down)
constant number_of_mins : natural := 99;
constant number_of_secs : natural := 59;

-- Clock period.
constant Tperiod : time := 10 ns;
constant small_delay : time := Tperiod/10;


begin

	-- Instantiate the kitchen timer.
	dut: kookwekker_counter
	port map (clk_32768 => sim_clk_32768, areset => sim_areset, mins_up => sim_mins_up,
				 secs_up => sim_secs_up, countdown => sim_countdown, clear => sim_clear,
				 tens_mins => sim_tens_mins, mins => sim_mins, tens_secs => sim_tens_secs,
				 secs => sim_secs, time_zero => sim_time_zero);

	-- The clock generator. It generates clock pulses until it is signaled to stop.
	clkgen: process is
	begin
		-- Loop until it's time to stop...
		while still_simulating loop
			sim_clk_32768 <= '0';
			wait for Tperiod/2;
			sim_clk_32768 <= '1';
			wait for Tperiod/2;
		end loop;
		
		-- Halt...
		wait;
	end process clkgen;
	
	datagen: process is
	-- Patern to apply to the mins_up and secs_up inputs.
	constant x_i_s : std_logic_vector := "1000000";
   variable x_i : std_logic_vector(x_i_s'range) := x_i_s;
	begin
	
		-- All to default values...
		sim_mins_up <= '0';
		sim_secs_up <= '0'; 
		sim_countdown <= '0';
		sim_clear <= '0';
	
		-- Reset the lot...
		sim_areset <= '1';
		wait for Tperiod*2;
		sim_areset <= '0';
		wait for Tperiod*2;
		wait until sim_clk_32768 = '1';
		wait for small_delay;

		-- Assign to mins_up
		for i in 1 to number_of_mins loop
			for index in x_i'range loop   -- loop through all input bits
				sim_mins_up <= x_i(index);        -- assign to FSM input
				wait until sim_clk_32768 = '1';
				wait for small_delay;
			end loop;
		end loop;
		
		-- Assign to secs_up
		for i in 1 to number_of_secs loop
			for index in x_i'range loop   -- loop through all input bits
				sim_secs_up <= x_i(index);        -- assign to FSM input
				wait until sim_clk_32768 = '1';
				wait for small_delay;
			end loop;
		end loop;
		
		-- Now reset the counter by asserting clear
		sim_clear <= '1';
		wait until sim_clk_32768 = '1';
		wait until sim_clk_32768 = '1';
		wait until sim_clk_32768 = '1';
		wait until sim_clk_32768 = '1';
		wait until sim_clk_32768 = '1';
		wait for small_delay;
		sim_clear <= '0';
		wait until sim_clk_32768 = '1';
		wait for small_delay;
		
		-- Assign to mins_up again
		for i in 1 to number_of_mins loop
			for index in x_i'range loop   -- loop through all input bits
				sim_mins_up <= x_i(index);        -- assign to FSM input
				wait until sim_clk_32768 = '1';
				wait for small_delay;
			end loop;
		end loop;
		
		-- Assign to secs_up again
		for i in 1 to number_of_secs loop
			for index in x_i'range loop   -- loop through all input bits
				sim_secs_up <= x_i(index);        -- assign to FSM input
				wait until sim_clk_32768 = '1';
				wait for small_delay;
			end loop;
		end loop;

		report "Starting count down..." severity note;
		
		-- Start countdown...
		sim_countdown <= '1';

		-- Wait for the timer/counter to reach zero...
		wait until sim_time_zero = '1';

		-- Stop countdown...
		sim_countdown <= '0';
		wait until sim_clk_32768 = '1';
		wait for small_delay;

		
		-- Signal the clockgen process to halt...
		still_simulating <= false;
		
		-- Halt ourselves...
		wait;
	end process datagen;

end architecture sim;